function inputs = ComputeInputsNew(x,A,b,partitions,region)

% this function implements a distributed version of the gradient projection
% method described in Luenberger's book

% x: matrix of robot locations (2 rows, n columns)
% A,b: matrix and vector definining linear constratins on x, i.e. for each
% column of x it must be Ax <= b

n_robots = size(x,2);

% compute gradient
gradient = ComputeGradient(x,partitions);

projected_gradient = gradient;

% for every robot find the active constraints (there are at most two)

active = zeros(n_robots,2);

for i = 1:n_robots
    %Vi = ExtractVertices(partitions{i,3});
    Vi = ContractPolygon(ExtractVertices(partitions{i,3}),region,x(:,i),0.01);
    c = find(abs(A*x(:,i)-b)<0.000001);
    if numel(c) == 2
        active(i,:) = c;
    elseif numel(c) == 1
        active(i,1) = c(1);
    else
        projected_gradient(:,i) = ComputeSecureInput(x(:,i),gradient(:,i),Vi);
    end
    if sum(active(i,:))>0 % projection needed
        while sum(active(i,:)>0) >= 1
            Aq = zeros(sum(active(i,:)>0),2);
            for j = 1:sum(active(i,:)>0)
                active(i,j)
                Aq(j,:) = A(active(i,j),:);
            end
            P = eye(2)-Aq'*inv(Aq*Aq')*Aq;
            d = P*gradient(:,i);
            if norm(d) > 0.001 % found direction
                projected_gradient(:,i) = ComputeSecureInput(x(:,i),d,Vi);
                active(i,:) = [0 0]; % teminate looping
            else % check if KKT conditions are verified
                l = -inv(Aq*Aq')*Aq*gradient(:,i);
                if sum(l<0) == sum(active(i,:)>0) % x satisfies KKT; stop
                    active(i,:) = [0 0];
                    projected_gradient(:,i) = [0 0]';
                else % eliminate one constraint
                    if l(1) > l(2)
                        active(i,:) = [active(i,2) 0];
                    else
                        active(i,2) = 0;
                    end
                end
            end
        end
    end
end


inputs = projected_gradient;