function f = ContractPolygon(Vi,region,p,epsilon)

%
%  partition: subset of region associated with p
%  region: boundaries of convex outer region (must be closed)
%  p: location of the robot that generated the partition
%  epsilon: amount of contraction
%


f = zeros(4,size(Vi,2)-1);

if max(region(:,1) ~= region(:,end)) > 0
    region = [region region(:,1)];
end

n_edges = size(Vi,2)-1;

for i = 1:n_edges
    edge = [Vi(:,i)' Vi(:,i+1)'];
    if IsOnBoundary(edge,region)
        f(:,i) = edge';
    else
        a = Shift(edge,p,epsilon);
        f(:,i) = a'; %Shift(edge,p,epsilon);
    end
end

f = Prune(f);
f = f(1:2,:);
f = [f f(:,1)]; % close the boundary

end

function b = IsOnBoundary(edge,region)

n_edges = size(region,2)-1;
b = 0;

for i = 1:n_edges
    if IsSubSegment(edge,[region(:,i)' region(:,i+1)'])
        b = 1;
        i=n_edges+1; %#ok<FXSET>
    end
end

end

function b = IsSubSegment(e1,e2)

if abs(e1(1)-e1(3)) < 10^-2
    slope1 = inf;
else
    slope1 = (e1(2)-e1(4))/(e1(1)-e1(3));
end

if abs(e2(1)-e2(3)) < 10^-2
    slope2 = inf;
else
    slope2 = (e2(2)-e2(4))/(e2(1)-e2(3));
end

if ( ( slope1 == inf ) && ( slope2 ~= inf ) ) || ...
        ( (  slope2 == inf ) && ( slope1 ~= inf ) )
    b = 0;
    return;
end

if  ( slope1 == inf ) && (slope2 == inf)
    if abs(e1(1)-e2(1)) < 10^-2
        if ( (  e1(2) >= e2(2) ) && ( e1(2) <= e2(4) ) ) || ...
                ( (  e1(2) >= e2(4) ) && ( e1(2) <= e2(2) ) )
            b = 1;
        else
            b = 0;
        end
    else
        b = 0;
    end
    return;
end

if ( abs(slope1-slope2) > 10^-2 )
    b = 0;
    return;
end

% if we got here the slope is the same

q1 = e1(2)-slope1*e1(1);
q2 = e2(2)-slope2*e2(1);

if ( abs(q1-q2) > 10^-2 )
    b = 0;
    return;
end

if  ( ( e1(1) >= e2(1) ) && ( e1(1) <= e2(3) ) ) || ...
        ( ( e1(1) >= e2(3) ) && ( e1(1) <= e2(1) ) )
    b = 1;
else
    b = 0;
end

end

function n = Shift(edge,p,epsilon)
if abs(edge(1)-edge(3)) < 10^-2 % vertical edge
    n = edge;
    if edge(1) > p(1)
        n(1) = edge(1)-epsilon;
        n(3) = edge(3)-epsilon;
    else
        n(1) = edge(1) + epsilon;
        n(3) = edge(3) + epsilon;
    end
    return;
end
slope = (edge(2)-edge(4))/(edge(1)-edge(3));
if abs(slope) < 10^-2
    n = edge;
    if edge(2) > p(2)
        n(2) = edge(2)-epsilon;
        n(4) = edge(4)-epsilon;
    else
        n(2) = edge(2)+epsilon;
        n(4) = edge(4)+epsilon;
    end
    return;
end

normal_slope = -1/slope;

alpha = atan(normal_slope);

deltax = epsilon * cos(alpha);
deltay = epsilon * sin(alpha);

n = edge - [deltax deltay deltax deltay];
if norm(n(1:2)'-p) > norm(edge(1:2)'-p)
    n = edge + [deltax deltay deltax deltay];
end
end


function f = Prune(f)

n_edges = size(f,2);

for i = 1:n_edges-1
    [coll, cc] = Myintersect(f(:,i)',f(:,i+1)');
    if ( coll > 0 )
        f(3:4,i) = cc(:);
        f(1:2,i+1) = cc(:);
    end
end

[coll, cc] = Myintersect(f(:,n_edges)',f(:,1)');
if ( coll > 0 )
    f(3:4,n_edges) = cc(:);
    f(1:2,1) = cc(:);
end

end