function [i,p] = Myintersect(s1,s2)

%
%  Given two segments s1,s2 returns i=0 if they do not intesect. If
%  they intersect, returns i=1, and p is the intersection points
%
%  s1,s2: segments (represented as row vectors, x,y,x,y)
%

epsilon = 0.000001;

i = 0;
p = [0;0];

if abs(s1(1)-s1(3)) < epsilon
    slope1 = inf;
elseif abs(s1(2)-s1(4)) < epsilon
    slope1 = 0;
else
    slope1 = ( s1(4)-s1(2) ) / ( s1(3)-s1(1) );
end
if ( slope1 ~= inf )
    q1 = s1(2) - slope1 * s1(1);
end


if abs(s2(1)-s2(3)) < epsilon
    slope2 = inf;
elseif abs(s2(2)-s2(4)) < epsilon
    slope2 = 0;
else
    slope2 = ( s2(4)-s2(2) ) / ( s2(3)-s2(1) );
end
if slope2 ~= inf
    q2 = s2(2) - slope2 * s2(1);
end

if ( slope1 == slope2 ) % parallel case: no intersections
    return;
    
elseif  ( slope1 ~= inf ) &&  ( slope2 ~= inf ) % general case; no vertical segments
    x = (q2-q1) / ( slope1-slope2);
    if ( ( ( s1(1) <= x ) && ( s1(3) >= x ) ) || ( ( s1(3) <= x ) && ( s1(1) >= x ) ) ) && ...% intersection detected;
            ( ( ( s2(1) <= x ) && ( s2(3) >= x ) ) || ( ( s2(3) <= x ) && ( s2(1) >= x ) ) )
        i = 1;
        p = [x; slope1*x + q1];
    end
elseif ( ( slope1 == inf ) && ( slope2 == 0 ) ) || ...
        ( ( slope2 == inf ) && ( slope1 == 0 ) ) % orthogonal case
    if slope2 == inf % makes s1 vertical and s2 horizontal
        tmp = s1;
        s1 = s2;
        s2 = tmp;
    end
    if s1(2) > s1(4) % make s1 from bottom to top
        s1 = [s1(1) s1(4) s1(3) s1(2)];
    end
    if s2(1) > s2(3) % make s2 from left to right
        s2 = [s2(3) s2(2) s2(1) s2(4)];
    end
    if (s1(2) <= s2(2) ) && ( s2(2) <= s1(4) ) && ( s2(1) <= s1(1) ) && ( s1(1) <= s2(3) )
        i = 1;
        p = [s1(1) ; s2(2) ];
    end
    
else % one of the two segments is vertical but the other is not
    if slope2 == inf % let s1 be vertical
        tmp = s1;
        s1 = s2;
        s2 = tmp;
        slope2 = slope1;
        q2 = q1;
    end
    if s1(2) > s1(4) % make s1 from bottom to top
        s1 = [s1(1) s1(4) s1(3) s1(2)];
    end
    if s2(1) > s2(3) % make s2 from "left to right"
        s2 = [s2(3) s2(4) s2(1) s2(2)];
    end
    if ( s2(1) <= s1(1) ) && ( s1(1) <= s2(3) )
        y = slope2*s1(1) + q2;
        if ( s1(2) <= y ) && ( y <= s1(4)  )
            i = 1;
            p = [s1(1);y];
        end
    end
    
end

end
