function out  = PruneSegments(f)

out = f;
epsilon = 0.01;

nsegments = size(f,1);

for i = 1:nsegments
    for j = 1:nsegments
        % check if segment i intersects segment j
        
        if ( i > j ) % do it only where needed
            
            if abs(f(i,1)-f(i,3)) < epsilon
                slopei = inf;
            elseif abs(f(i,2)-f(i,4)) < epsilon
                slopei = 0;
            else
                slopei = ( f(i,4)-f(i,2) ) / ( f(i,3)-f(i,1) );
            end
            if ( slopei ~= inf )
                qi = f(i,2) - slopei * f(i,1);
            end
            
            
            if abs(f(j,1)-f(j,3)) < epsilon
                slopej = inf;
            elseif abs(f(j,2)-f(j,4)) < epsilon
                slopej = 0;
            else
                slopej = ( f(j,4)-f(j,2) ) / ( f(j,3)-f(j,1) );
            end
            if slopej ~= inf
                qj = f(j,2) - slopej * f(j,1);
            end
            
            if ( slopei == slopej ) % parallel case: no intersections
                continue
            elseif  ( slopei ~= inf ) &&  ( slopej ~= inf ) % general case
                %                 i
                %                 slopei
                %                 slopej
                %                 j
                
                x = (qj-qi) / ( slopei-slopej);
                if ( ( ( f(i,1) < x ) && ( f(i,3) > x ) ) || ( ( f(i,3) < x ) && ( f(i,1) > x ) ) ) && ...% intersection detected; pruning
                        ( ( ( f(j,1) < x ) && ( f(j,3) > x ) ) || ( ( f(j,3) < x ) && ( f(j,1) > x ) ) )
                    % prune segment
                    if  ( f(j,5) ~= 0 ) && ( f(i,5) ~= 0 )
                        %if f(j,5) ~= 0
                        if instripe(f(i,1),f(i,2),f(j,:),f(f(j,5),:))
                            out(i,1) = x;
                            out(i,2) = slopei*x + qi;
                        else
                            out(i,3) = x;
                            out(i,4) = slopei*x + qi;
                        end
                        %end
                        % prune segment j
                        %if f(i,5) ~= 0
                        if instripe(f(j,1),f(j,2),f(i,:),f(f(i,5),:))
                            out(j,1) = x;
                            out(j,2) = slopej*x + qj;
                        else
                            out(j,3) = x;
                            out(j,4) = slopej*x + qj;
                        end
                        % end
                    end
%                     line([out(i,1) out(i,3)], [out(i,2) out(i,4)] , 'Marker','.','LineStyle','-','Linewidth',4,'Color','r');
%                     line([out(j,1) out(j,3)], [out(j,2) out(j,4)] , 'Marker','.','LineStyle','-','Linewidth',4,'Color','r');
                end
            else % one of the two slopes is infinite (but not both!) % THIS PART HAS NOT BEEN TESTED YET
                if slopei == inf
                    x = f(i,1);
                else % then slopej must be inf
                    x = f(j,1);
                end
                
                if ( ( ( f(i,1) < x ) && ( f(i,3) > x ) ) || ( ( f(i,3) < x ) && ( f(i,1) > x ) ) ) && ...% intersection detected; pruning
                        ( ( ( f(j,1) < x ) && ( f(j,3) > x ) ) || ( ( f(j,3) < x ) && ( f(j,1) > x ) ) )
                    % prune segment
                    if  ( f(j,5) ~= 0 ) && ( f(i,5) ~= 0 )
                        %if f(j,5) ~= 0
                        if instripe(f(i,1),f(i,2),f(j,:),f(f(j,5),:))
                            out(i,1) = x;
                            out(i,2) = slopei*x + qi;
                        else
                            out(i,3) = x;
                            out(i,4) = slopei*x + qi;
                        end
                        %end
                        % prune segment j
                        %if f(i,5) ~= 0
                        if instripe(f(j,1),f(j,2),f(i,:),f(f(i,5),:))
                            out(j,1) = x;
                            out(j,2) = slopej*x + qj;
                        else
                            out(j,3) = x;
                            out(j,4) = slopej*x + qj;
                        end
                        % end
                    end
%                     line([out(i,1) out(i,3)], [out(i,2) out(i,4)] , 'Marker','.','LineStyle','-','Linewidth',4,'Color','r');
%                     line([out(j,1) out(j,3)], [out(j,2) out(j,4)] , 'Marker','.','LineStyle','-','Linewidth',4,'Color','r');
                end
                
            end
            
            
        end
        
    end
end


function b = instripe(x,y,s1,s2)

slope1 = ( s1(4)-s1(2) ) / ( s1(3)-s1(1) );
q1 = s1(2) - slope1*s1(1);


slope2 = ( s2(4)-s2(2) ) / ( s2(3)-s2(1) );
q2 = s2(2) - slope2*s2(1);

y1 = x*slope1 + q1;
y2 = x*slope2 + q2;

if ( ( y1 < y ) && ( y < y2 ) ) || ( ( y2 < y ) && ( y < y1 ) )
    b = true;
else
    b = false;
end